require "spreadsheet_accessor/version"
require "spreadsheet"

class SpreadsheetAccessor
	attr_accessor	:sheet, :cells
	
	@@col_start_index	= 65 # A
	@@alphabet_size		= 26
	
	def self.cell( name )
		row = 0
		col = 0
		
		name.upcase!
		if /^([A-Z]+)/ =~ name
			bytes = $1.bytes
			col = bytes.pop - @@col_start_index
			bytes.reverse.each_with_index{|byte, i|
				value = byte - @@col_start_index + 1
				( i + 1 ).times{
					value *= @@alphabet_size
				}
				col += value
			}
		end
		
		row = $1.to_i - 1 if /([0-9]+)$/ =~ name
		
		{
			:name	=> name,
			:row	=> row,
			:col	=> col
		}
	end
	
	def self.index( data )
		row = 0
		col = 0
		
		if data.instance_of?( String )
			hash = SpreadsheetAccessor.cell( data )
			row = hash[ :row ]
			col = hash[ :col ]
		elsif data.instance_of?( Array )
			row = data[ 0 ]
			col = data[ 1 ]
		elsif data.instance_of?( Hash )
			row = data[ :row ] if data.key?( :row )
			col = data[ :col ] if data.key?( :col )
		end
		
		{
			:row	=> row,
			:col	=> col
		}
	end
	
	def initialize( sheet = nil )
		self.sheet = sheet
	end
	
	def sheet=( sheet )
		@sheet = sheet
		@cells = sheet
	end
	
	def cell( data )
		hash = SpreadsheetAccessor.index( data )
		cell = @cells[ hash[ :row ], hash[ :col ] ]
		cell.instance_of?( Spreadsheet::Formula ) ? cell.value : cell
	end
end
